<?php
/**
 * config_functions - Functions for the value handling. Function names are the variable names.
 * 						 A function is not required for any variable if not required.
 *
 * @author $Author: dtong $
 * @version $Id: config_functions.php,v 1.14 2011/11/07 07:51:39 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/
require_once 'config_data.php';

class config_functions {
	private $CI;
	//These functions has to be unique strings in the config variables.
	//Prefix functions are ones matching the prefix substring of a parameter.
	private $prefix_functions = array('cms_template_group_');
	private $function = FALSE;
	public $error_msg = FALSE;
	//Right now just STATUS_DISABLE, greys out the words and no MODIFY link
	public $last_status = FALSE;
	public $edit_save_jscript = FALSE;

	function __construct() {
		$this->CI = &get_instance();
	}

	/*
	 * Interface to all functions for getting and setting the value
	 * READ action should preprocess the value
	 * WRITE and DELETE actions should process extra code needed, need to supply the new value
	 *
	 * A retun of FALSE from a function will cause the row not to display
	 */
	function value_handle($function, $value, $action=config_data::ACTION_READ) {
		$this->function = $function;
		$this->error_msg = FALSE;
		$this->last_status = FALSE;
		$this->edit_save_jscript = FALSE;
		if ( method_exists($this, $function) ) {
			return $this->$function($value, $action);
		}
		foreach ($this->prefix_functions as $pfunc) {
			if ( strpos($function, $pfunc) === 0 ) {
				return $this->$pfunc($value, $action);
			}
		}
		return $value;
	}

	/*
	 * This is a prefix function. This handles all config parameters from the template config.
	 * This is different than the system configs where each option is a static config parameter.
	 * The template configs are all dynamic except the 2 system TODCM templates. The design is to support
	 * any number of customized templates. None od these template config params are get written to the DB.
	 * This uses a config override file instead.
	 */
	private function cms_template_group_($value, $action) {
		$template_group = osa_str_extract($this->function, 'cms_template_group_', FALSE);
		if ($action == config_data::ACTION_READ) {
			//Runs this jscript when admin clicks save on the edit popup. This refreshes the menu panel.
			//Other code will get this too but not used.
			$this->edit_save_jscript = "refresh_menu_panel('" . site_url() . "menu/menu_tree/index/admin')";
		}
		if ( $value ) {
			if ( $this->CI->config->item('cms_unit_org') != $template_group ) {
				//We know the admin has chosen a new template here
				$filetext = "<?php\n//Generated from admin config. DO NOT EDIT.\n\$config['cms_unit_org']='{$template_group}';\n";
				$config_dir = $this->CI->config->item('cms_data_folder') . 'config/';
				if ( !is_dir($config_dir) ) {
					@mkdir($config_dir);
				}
				//Add the config override for the new template group
				$ret = @file_put_contents($config_dir . 'config_override_' . $this->CI->config->item('cms_siteid') . EXT, $filetext);
				if ( !osa_is_int_one($ret) ) {
					echo osa_ajaxmsg(sprintf(lang('syscfg_template_file_access_error'), $this->CI->config->item('cms_data_folder')));
					exit;
				}
			}
			//Change the cms_unit_org so the config values are displayed properly
			$this->CI->config->set_item('cms_unit_org', $template_group);
		}
		if ( $this->CI->config->item('cms_unit_org') == $template_group ) {
			//We are processing the chosen template group
			if ( $template_group == CMS_TEMPLATE_NAME_TODCM2 && $action == config_data::ACTION_PREWRITE) {
				//Do todcm2 specific data processing if we just switched over to it
				require_once 'config_util_todcm2.php';
				$todcm2_config_obj = $this->CI->cms_cache->object_get('config_util_todcm2');
				//Add the default starnd and standard id configs. These are never deleted even if chnaging the templaste group.
				$this->CI->load->model('admin/config_model');
				$this->CI->config_model->update(config_data::SITE_TEMPLATE,
					'cms_outcomes_skills_strandid', $todcm2_config_obj->strandid());
				$this->CI->config_model->update(config_data::SITE_TEMPLATE,
					'cms_outcomes_skills_standardid', $todcm2_config_obj->standardid());
			}
			$this->last_status = config_data::STATUS_DISABLE_BOLD;
			//No need to write anything to the DB since this is a file based configuration
			if ( $action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_PREDELETE) {
				$this->last_status = config_data::STATUS_SKIP_DB_UPDATE;
			}
			return TRUE;
		}
		return FALSE;
	}

	//Put blank value if all unit types is set
	private function cms_use_unit_unified($value, $action) {
		if ( $action == config_data::ACTION_READ ) {
			//Disable if cms_all_unittypes is true
			if ( $this->CI->config->item('cms_use_all_unit_types') ) {
				$this->last_status = config_data::STATUS_BLANK_VALUE;
				//$this->last_status = config_data::STATUS_SKIP;
			}
		}
		return $value;
	}

	private function cms_use_unit_integrated($value, $action) {
		return $this->cms_use_unit_unified($value, $action);
	}

	private function cms_use_unit_meta($value, $action) {
		return $this->cms_use_unit_unified($value, $action);
	}

	//Clear sessions on write and refresh on new read
	private function cms_manage_mode($value, $action) {
		switch ($action) {
			case config_data::ACTION_POSTWRITE:
				//Maintenace mode check is now in the startup library
				/*if ( $value == TRUE ) {
					osa_clear_ci_sessions();
				}*/
				break;
			case config_data::ACTION_READ:
				$this->last_status = config_data::STATUS_REFRESH_PAGE;
				break;
		}
		return $value;
	}

	private function cms_header_message($value, $action){
		return $this->cms_header_image_bkcolor($value, $action);
	}

	//Do refresh on new read
	private function cms_header_image_bkcolor($value, $action) {
		if ($action == config_data::ACTION_READ) {
			$this->last_status = config_data::STATUS_REFRESH_PAGE;
		}
		return $value;
	}

	private function cms_header_image_left_align($value, $action) {
		return $this->cms_header_image_bkcolor($value, $action);
	}

	//Put blank value if not using category
	private function cms_category_not_show_empty_departments($value, $action) {
		if ($action == config_data::ACTION_READ && !$this->CI->config->item('cms_menutree_use_category') ) {
			$this->last_status = config_data::STATUS_BLANK_VALUE;
		}
		return $value;
	}

	private function cms_header_image($value, $action) {
		return $this->cms_header_image_bkcolor($value, $action);
	}

	private function cms_failed_login_lockout_check($value, $action) {
		//Always clear the count when option is disabled.
		if ( $action == config_data::ACTION_POSTWRITE && $value == FALSE ) {
			$this->CI->load->model('auth/login_model');
			$this->CI->login_model->clear_failedlogin_count();
		}
		// force to update in config model
		if ( $action == config_data::ACTION_READ || $action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_PREDELETE ) {
			$this->CI->load->model('admin/config_model');
			if ( $value == TRUE ) {
				$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_login_use_captcha', FALSE);
				$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_failed_login_lockout_check', TRUE);
				// update in db isn't efficient, so set the same result in config
				$this->CI->config->set_item('cms_login_use_captcha', FALSE);
				$this->CI->config->set_item('cms_failed_login_lockout_check', TRUE);
			}
			else if ( $value == FALSE ){
				$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_login_use_captcha', TRUE);
				$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_failed_login_lockout_check', FALSE);
				// update in db isn't efficient, so set the same result in config
				$this->CI->config->set_item('cms_login_use_captcha', TRUE);
				$this->CI->config->set_item('cms_failed_login_lockout_check', FALSE);
			}
		}
		// disable function
		if ( $this->CI->config->item('cms_login_use_captcha') && !$this->CI->config->item('cms_failed_login_lockout_check') ) {
			$this->last_status = config_data::STATUS_BLANK_VALUE;
		}
		return $value;
	}

	private function cms_login_use_captcha($value) {
		if ( $this->CI->config->item('cms_failed_login_lockout_check') ) {
			$this->last_status = config_data::STATUS_BLANK_VALUE;
			return $value = FALSE;
		}
		if ( $value == FALSE && $this->CI->config->item('cms_failed_login_lockout_check') == FALSE ) {
			$this->cms_failed_login_lockout_check(TRUE, config_data::ACTION_PREWRITE);
		}
		return $value;
	}

	/*
	private function cms_use_all_unit_types($value, $action) {
		switch ($action) {
			case config_data::ACTION_POSTWRITE:
				if ( $value ) {
					//If turning all unit types to true then make all the types to be true too
					$this->CI->load->model('admin/config_model');
					$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_use_unit_unified', TRUE);
					$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_use_unit_integrated', TRUE);
					$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_use_unit_meta', TRUE);
				}
				break;
			//No need for all these cases but just to show them since this is the first real handle
			case config_data::ACTION_PREDELETE:
			case config_data::ACTION_POSTDELETE:
			case config_data::ACTION_PREWRITE:
			case config_data::ACTION_READ:
			default:
				break;
		}
		//Only the read action really needs the return value.
		return $value;
	}
	*/

	private function cms_max_uploadsize($value, $action){
		if ( $action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_READ ) {
			// cms_max_uploadsize uses megabytes unit but osa_maximum_uploadsize return bytes
			// so we must convert megabytes to bytes unit
			$megabytes2bytes = 1048576; // 1024 * 1024
			// $max_upload value will be megabytes unit.
			$max_upload = osa_maximum_uploadsize() / $megabytes2bytes;
			if ( $value < 1 || $max_upload < 1 ) {
				return 1; // upload size ALWAYS in megabyte so if value less than 1, set it to 1 (minimum value)
			}
			if ( $value > $max_upload ){
				return $max_upload;
			}
		}
		return $value;
	}

	private function cms_failed_lockout_count($value, $action){
		$minimum_counts = 3;
		if ( ($action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_READ) && $value < $minimum_counts ) {
			$value = $minimum_counts;
		}
		return $value;
	}

	private function cms_lockout_period($value, $action){
		$minimum_minutes = 5;
		if ( ($action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_READ) && $value < $minimum_minutes ) {
			$value = $minimum_minutes;
		}
		return $value;
	}

	private function cms_survey_nologin($value, $action){
		// check action READ because user can disable this item in config file
		// so if it's false, enable another one.
		if ( $value == FALSE && ( $action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_READ ) ) {
			$this->CI->load->model('admin/config_model');
			$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_survey_uselogin', TRUE);
			$this->CI->config->set_item('cms_survey_uselogin', TRUE);
		}
		return $value;
	}

	private function cms_survey_uselogin($value, $action){
		// check action READ because user can disable this item in config file
		// so if it's false, enable another one.
		if ( $value == FALSE && ( $action == config_data::ACTION_PREWRITE || $action == config_data::ACTION_READ ) ) {
			$this->CI->load->model('admin/config_model');
			$this->CI->config_model->update(config_data::SYSTEM_GENERAL, 'cms_survey_nologin', TRUE);
			$this->CI->config->set_item('cms_survey_nologin', TRUE);
		}
		return $value;
	}

}